/**
 * PieChart.js
 * ------------
 * Implements the createPieChart function and a short test
 * framework to demonstrate it works properly.
 */

"use strict";

const GWINDOW_WIDTH = 300;
const GWINDOW_HEIGHT = 200;
const WEDGE_COLORS = [
   "Red", "Yellow", "Blue", "Green", "Pink", "Cyan"
];

/**
 * Function: createPieChart
 * ------------------------
 * Creates and returns a GCompound object that represents a pie
 * chart.  Each value in the supplied data array relative to the
 * sum of all values dictates the size of each slice in the pie
 * chart.  The reference point of the entire GCompound is the pie
 * chart's center.
 */
function createPieChart(r, data) {
	let pie = GCompound();
	let total = sumArray(data);
	let start = 0;
	for (let i = 0; i < data.length; i++) {
		let fraction = data[i] / total;
		let sweep = fraction * 360;
		let slice = GArc(-r, -r, 2 * r, 2 * r, start, sweep);
		slice.setFilled(true);
		slice.setFillColor(WEDGE_COLORS[i % WEDGE_COLORS.length]);
		pie.add(slice);
		start += sweep;
	}
	return pie;
}

/**
 * Function: sumArray
 * ------------------
 * Returns the sum of all the numbers residing
 * in the supplied array. (This function would not
 * need to be written out, since it appears in a
 * lecture slide and in the reader, on page 258.)
 */
function sumArray(array) {
	let sum = 0;
	for (let i = 0; i < array.length; i++) {
		sum += array[i];
	}
	return sum;
}

/**
 * Function: TestPieChart
 * ----------------------
 * Exercises the createPieChart functionality implemented by the
 * functions above.
 */
function TestPieChart() {
   let gw = GWindow(GWINDOW_WIDTH, GWINDOW_HEIGHT);
   let data = [ 45, 25, 15, 10, 5 ];
   let pieChart = createPieChart(50, data);
   gw.add(pieChart, gw.getWidth()/2, gw.getHeight()/2);
}



