/**
 * File: Anagram.js
 * ----------------
 * This file defines the isAnagram function and a small
 * test harness to confirm it works properly.
 */

"use strict";

/**
 * Predicate Function: isAnagram
 * -----------------------------
 * Returns true if and only if the two supplied
 * strings are anagrams, as per the definition of
 * anagram in the problem statement.
 */
function isAnagram(s1, s2) {
	let table1 = compileFrequencyTable(s1);
	let table2 = compileFrequencyTable(s2);
	for (let i = 0; i < 26; i++) {
		if (table1[i] !== table2[i])
			return false
	}
	return true;
}

/**
 * Function: compileFrequencyTable
 * -------------------------------
 * Compiles and returns a frequency table for the
 * supplied string.  The table is of length 
 * 26, and the 0th entry ultimately contains the number
 * of a's (and A's), the 1th entry ultimately contains the
 * number of b's and B's, and so forth.
 */
function compileFrequencyTable(str) {
	let table = createArray(26, 0);
	for (let i = 0; i < str.length; i++) {
		if (isLetter(str.charAt(i))) {
			let ch = str.charAt(i).toUpperCase();
			table[ch.charCodeAt(0) - "A".charCodeAt(0)]++;
		}
	}
	return table;
}

/**
 * Predicate Function: isLetter
 * ----------------------------
 * Returns true if and only if ch is of length one, and its
 * one character is a letter of the English alphabet.  This
 * function did not need to be written out, since it
 * appears on page 239, and has appeared in several lecture slides.
 */ 
function isLetter(ch) {
	if (ch.length !== 1) return false;
	ch = ch.toUpperCase();
	return ch >= "A" && ch <= "Z";
}

/**
 * Function: createArray
 * ---------------------
 * Creates an array of the specified length where
 * every single element is equal to the supplied value.
 * This function did not need to be written, since it
 * appears on page 263 of the course reader, and it's also
 * appeared in one or more lecture slides.
 */
function createArray(length, value) {
	let array = [];
	for (let i = 0; i < length; i++) {
		array.push(value);
	}
	return array;
}

/**
 * Function: Anagram
 * -----------------
 * Confirms that our isAnagram function works as expected.
 */
function Anagram() {
	console.log("The following calls to isAnagram should all return true.");
	console.log("  isAnagram(\"\", \"\") --> " + isAnagram("", ""));
	console.log("  isAnagram(\"same\", \"same\") --> " + isAnagram("same", "same"));
	console.log("  isAnagram(\"listen\", \"silent\") --> " + isAnagram("listen", "silent"));
	console.log("  isAnagram(\"rail safety\", \"fairy tales\") --> " + isAnagram("rail safety", "fairy tales"));
	console.log("  isAnagram(\"I am a weakish speller\", \"William Shakespeare\") --> " + isAnagram("I am a weakish speller", "William Shakespeare"));
	console.log("The following calls to isAnagram should all return false.");
	console.log("  isAnagram(\"listen\", \"listens\") --> " + isAnagram("listen", "listens"));
	console.log("  isAnagram(\"silent\", \"listens\") --> " + isAnagram("silent", "listens"));
}
