/**
 * File: DoubleImage.js
 * --------------------
 * This file defines the doubleImage function as
 * simple test harness to confirm it really works.
 */

"use strict";

const WINDOW_WIDTH = 1200;
const WINDOW_HEIGHT = 600;

/**
 * Function: DoubleImage
 * ---------------------
 * Defines the test harness that allows the user to select an
 * image (assumed to be small enough to fit in the graphics window) and
 * then lays the selected image down in the left half of the window
 * and the doubled image in the right half of the window.
 */
function DoubleImage() {
	JSFileChooser.chooseDataFile(function(dataURL) {
		let gw = GWindow(WINDOW_WIDTH, WINDOW_HEIGHT);
		let original = new GImage(dataURL);
		original.addEventListener("load", function() {
			gw.add(original, WINDOW_WIDTH/4 - original.getWidth()/2, 
		                 	 WINDOW_HEIGHT/2 - original.getHeight()/2);
			let doubled = doubleImage(original);
			doubled.addEventListener("load", function() {
				gw.add(doubled, 3 * WINDOW_WIDTH/4 - doubled.getWidth()/2, 
		                            WINDOW_HEIGHT/2 - doubled.getHeight()/2);
			});
		});
	});
}

/**
 * Function: doubleImage
 * ---------------------
 * Accepts the fully loaded image, constructs a replica
 * of that image, save for the fact that it's twice as wide and
 * twice as tall, and then returns that image.
 */
function doubleImage(image) {
	let compact = image.getPixelArray();
	let expanded = [];
	for (let row = 0; row < compact.length; row++) {
		// introduce two new rows in expanded on behalf of one in original
		expanded.push([]); 
		expanded.push([]);
		for (let column = 0; column < compact[0].length; column++) {
			// append two pixels in each of the two new rows on behalf of compact[row][column]
			let pixel = compact[row][column];
			expanded[2 * row].push(pixel);
			expanded[2 * row].push(pixel);
			expanded[2 * row + 1].push(pixel);
			expanded[2 * row + 1].push(pixel);
		}
	}
	return GImage(expanded);
}
